/*--------------------------------------------------------------*
 *	Coupled Resonator Bandpass Filter Program		*
 *	Translated from a Terry Hock's BASIC program		*
 *	which is derived from the equations from Hayward's	*
 *	book on Page 92						*
 *								*
 *	G. Forrest Cook - 10/26/1988, www.solorb.com		*
 *	License: GPL Version 3					*
\*--------------------------------------------------------------*/

# include <stdio.h>
# include <math.h>
# define TRUE 1
# define FALSE 0

/*
 * A few function declarations
 */
int get_g_num ();
void set_imped ();
int bpf ();
void file_out ();
void set_induct ();
void set_freq ();
void set_type ();
int pr_menu1 ();
float f_sqrt ();
float f_log10 ();


/*
 * Lots of global vars, it WAS originally written in BASIC.
 */

float Fc, BW, Il, W, K, K12, Rpe;
float Co, Cm, Cei, Ceo, C1, C2;
float Qe, Qf, Qq, Qqo;
float picons = 6.284;			/* 2 * pi */
float micro = 0.000001;			/* 1 * 10^-6 */
float m = 1000000.0;			/* 1 * 10^6 */
float mm = 1000000000000.0;		/* 1 * 10^12 */
float F1, F2, tmp;
float L, Roi, Roo, Qu; 
int rv = FALSE, type = 1;
static float nk[] = {0.0,0.7071,0.7107,0.7154,0.7225,0.7290,0.7040,0.7466};
static float nq[] = {0.0, 1.414, 1.638, 1.779, 1.950, 2.091, 2.317, 2.452};
char *ft[] = {"", "Butterworth     ",
		  "Chebychev 0.1db ", "Chebychev 0.25db",
		  "Chebychev 0.5db ", "Chebychev 0.75db",
		  "Chebychev 1.0db ", "Chebychev 1.5db ", "\0"};

int main ()
{
	int sel;

	do
	{
		sel = pr_menu1 ();		/* display menu */

		switch (sel)
		{
		case 1:
			set_type ();
			set_freq ();
			set_induct ();
			set_imped ();
		break;
		case 2:
			set_type ();
		break;
		case 3:
			set_freq ();
		break;
		case 4:
			set_induct ();
		break;
		case 5:
			set_imped ();
		break;
		case 6:
			file_out (rv);
		break;
		}

		if ((sel > 0) && (sel < 6)) rv=bpf();
	}
	while (sel > 0);

	return (-1);
}

/*--------------------------------------------------------------*/

int pr_menu1 ()
{
	printf ("\n");
	printf ("\n0 -- Quit");
	printf ("\n1 -- Set All Parameters");
	printf ("\n2 -- Set Filter Type");
	printf ("\n3 -- Set Frequencies");
	printf ("\n4 -- Set Inductor");
	printf ("\n5 -- Set I/O Impedances");
	printf ("\n6 -- Save Info in a File");
	printf ("\n\nBandpass Filter Program Selection: ");
	return (get_g_num (0, 6, 1, 10));
}

/*--------------------------------------------------------------*/

void set_type ()
{
	int i = 1;

	printf ("\nFilter Type, Passband Ripple:\n");
	while (*ft[i])
	{
		printf ("%1d=%s  ", i, ft[i]);
		if (i++%3 == 0) printf ("\n");
	}

	printf ("\n\nSelect Type: ");
	type = get_g_num (1, i-1, 1, 10);
}

/*--------------------------------------------------------------*/

void set_freq ()
{
	printf ("\nEnter Lower Frequency (Mhz): ");
	scanf ("%f", &F1);
	printf ("\nEnter Upper Frequency (Mhz): ");
	scanf ("%f", &F2);
	getchar();	/* get last cr */
/*
 * swap frequencies if they are entered backwards
 */
	if (F1 > F2) { tmp = F1; F1 = F2; F2 = tmp; }
}

/*--------------------------------------------------------------*/

void set_induct ()
{
	printf ("\nEnter Resonator Inductance (uH): ");
	scanf ("%f", &L);
	printf ("\nEnter Inductor Unloaded Q: ");
	scanf ("%f", &Qu);
	getchar();	/* get last cr */
}

/*--------------------------------------------------------------*/

void set_imped ()
{
	printf ("\nEnter Input Impedance: ");
	scanf ("%f", &Roi);
	printf ("\nEnter Output Impedance: ");
	scanf ("%f", &Roo);
	getchar();	/* get last cr */
}

/*--------------------------------------------------------------*/

int bpf ()
{
	int ok = TRUE;

	Qq = nq[type];
	K = nk[type];
	BW = F2 - F1;			/* bandwidth */
	Fc = f_sqrt (F1 * F2);		/* geometric center freq */
	W = picons * Fc * m;
	Qf = Fc / (BW);			/* Loaded Q */
	Co = 1 / (W * W * L * micro);
	Qqo = Qu / Qf;
	Qe = 1 / ((1 / (Qq * Qf)) - (1 / Qu));
	K12 = K / Qf;
	Il = -20.0 * f_log10 (1 - (Qf / Qu)); /* insertion loss (db) */
	Cm = K12 * Co;
	Rpe = Qe * W * L * micro;

	if (Roi > Rpe)
	{
	    printf ("\nThe input impedance is out of range.\n");
	    ok = FALSE;
	}
	if (Roo > Rpe)
	{
	    printf ("\nThe output impedance is out of range.\n");
	    ok = FALSE;
	}


	if (ok)
	{
	    Cei = 1 / (W * f_sqrt ((Rpe * Roi) - (Roi * Roi)));
	    Ceo = 1 / (W * f_sqrt ((Rpe * Roo) - (Roo * Roi)));

	    C1 = Co - Cei - Cm;
	    C2 = Co - Ceo - Cm;

	    printf ("\n    SerC1          SerC2           SerC3");
	    printf ("\n0----||---+----+----||----+----+----||----0");
	    printf ("\n          |    |          |    |");
	    printf ("\n      ShL1(    -ShC1  ShL2(    -ShC2");
	    printf ("\nSource    (    -          (    -       Load");
	    printf ("\n          |    |          |    |");
	    printf ("\n0---------+----+----------+----+----------0");
	    printf ("\n\nFilter Type: %s", ft[type]);
	    printf ("\nBandwidth: %5.3f Mhz", BW);
	    printf ("\n3db points: %5.3f Mhz and %5.3f Mhz", F1, F2);
	    printf ("\nGeometric Center Frequency: %5.3f Mhz", Fc);
	    printf ("\nInsertion loss = %5.2f db, Qqo = %5.2f", Il, Qqo);
	    printf ("\nSource Impedance   = %5.2f ohms.", Roi);
	    printf ("\nSeries Capacitor 1 = %5.2f pF.", Cei * mm);
	    printf ("\nShunt  Capacitor 1 = %5.2f pF.", C1 * mm);
	    printf ("\nShunt  Inductance  = %5.2f uH.", L);
	    printf ("\nSeries Capacitor 2 = %5.2f pF.", Cm * mm);
	    printf ("\nShunt  Capacitor 2 = %5.2f pF.", C2 * mm);
	    printf ("\nShunt  Inductance  = %5.2f uH.", L);
	    printf ("\nSeries Capacitor 3 = %5.2f pF.", Ceo * mm);
	    printf ("\nLoad   Impedance   = %5.2f ohms.", Roo);

	    if (C1 < 0.0 || C2 < 0.0 || Cm < 0.0) 
	    {
	        printf("\n\nInvalid inductor for this filter, try again!\n");
	        ok = FALSE;
	    }
	}
	return (ok);
}

/*--------------------------------------------------------------*/

void file_out (int ok)	/* store the filter info in a file */
{
	static char *fn = {"bpf.dat"};
	FILE *fopen(), *fp;

	if (C1 < 0.0 || C2 < 0.0 || Cm < 0.0 || !ok)
	{
	  printf ("\n\nInvalid filter info, file not saved.");
	}
	else
	{
	  printf ("\n\nSaving filter information in file \"%s\".", fn);

	  fp = fopen (fn, "w");

	  fprintf (fp, "\n    SerC1          SerC2           SerC3");
	  fprintf (fp, "\n0----||---+----+----||----+----+----||----0");
	  fprintf (fp, "\n          |    |          |    |");
	  fprintf (fp, "\n      ShL1(    -ShC1  ShL2(    -ShC2");
	  fprintf (fp, "\nSource    (    -          (    -       Load");
	  fprintf (fp, "\n          |    |          |    |");
	  fprintf (fp, "\n0---------+----+----------+----+----------0");
	  fprintf (fp, "\n\nFilter Type: %s", ft[type]);
	  fprintf (fp, "\nBandwidth: %5.3f Mhz", BW);
	  fprintf (fp, "\n3db points: %5.3f Mhz and %5.3f Mhz", F1, F2);
	  fprintf (fp, "\nGeometric Center Frequency: %5.3f Mhz", Fc);
	  fprintf (fp, "\nInsertion loss = %5.2f db, Qqo = %5.2f", Il, Qqo);
	  fprintf (fp, "\nSource Impedance   = %5.2f ohms.", Roi);
	  fprintf (fp, "\nSeries Capacitor 1 = %5.2f pF.", Cei * mm);
	  fprintf (fp, "\nShunt  Capacitor 1 = %5.2f pF.", C1 * mm);
	  fprintf (fp, "\nShunt  Inductance  = %5.2f uH.", L);
	  fprintf (fp, "\nSeries Capacitor 2 = %5.2f pF.", Cm * mm);
	  fprintf (fp, "\nShunt  Capacitor 2 = %5.2f pF.", C2 * mm);
	  fprintf (fp, "\nShunt  Inductance  = %5.2f uH.", L);
	  fprintf (fp, "\nSeries Capacitor 3 = %5.2f pF.", Ceo * mm);
	  fprintf (fp, "\nLoad   Impedance   = %5.2f ohms.", Roo);

	  fclose (fp);
	}
}

/*--------------------------------------------------------------*/

float f_sqrt (num)
float num;
{
	return (float)sqrt((double)num);
}

/*--------------------------------------------------------------*/

float f_log10 (num)
float num;
{
	return (float)log10((double)num);
}

/*--------------------------------------------------------------*/
